@@ -18,8 +18,9 @@ from django.utils.safestring import SafeString
 from typing_extensions import TypeAlias
 
 from drip.exceptions import MessageClassNotFound
-from drip.models import Drip, SentDrip
-from drip.utils import build_now_from_timedelta, get_conditional_now, get_user_model
+from drip.models import Drip, SentDrip, UserUnsubscribe
+from drip.tokens import EmailToken
+from drip.utils import build_now_from_timedelta, get_conditional_now, get_user_model, validate_path_existence
 
 User = get_user_model()
 
@@ -127,9 +128,27 @@ class DripMessage(object):
     @property
     def context(self) -> Context:
         if not self._context:
-            self._context = Context({"user": self.user})
+            context_dict = self.build_context()
+            self._context = Context(context_dict)
         return self._context
 
+    def build_context(self) -> Dict[str, Any]:
+        """
+        Build context drip dictionary allowing you easy extension.
+        Also it is used to manage unsubscribe links configurations.
+        """
+        context = {"user": self.user}
+        unsubscribe_users = getattr(
+            settings,
+            "DRIP_UNSUBSCRIBE_USERS",
+            False,
+        )
+        if unsubscribe_users:
+            context["unsubscribe_link_drip"] = self._get_unsubscribe_link_drip()
+            context["unsubscribe_link_campaign"] = self._get_unsubscribe_link_campaign()
+            context["unsubscribe_link"] = self._get_unsubscribe_link()
+        return context
+
     @property
     def subject(self) -> SafeString:
         drip_subject: SafeString
@@ -187,6 +206,41 @@ class DripMessage(object):
                 self._message.attach_alternative(self.body, "text/html")
         return self._message
 
+    def _get_unsubscribe_link_drip(self) -> Optional[str]:
+        """
+        Generate url for Unsubscribe Drip with drip and user data and validates existence of this url in project.
+        Checking if it was configured by the user.
+        """
+        email_token = EmailToken(self.user)
+        drip_uidb64, uidb64, token = email_token.get_uidb64_token(self.drip_base.drip_model.pk)
+        url_args = {"drip_uidb64": drip_uidb64, "uidb64": uidb64, "token": token}
+        unsubscribe_link = validate_path_existence("unsubscribe_drip", url_args)
+        return unsubscribe_link
+
+    def _get_unsubscribe_link_campaign(self) -> Optional[str]:
+        """
+        Generate url for Unsubcribe Campaign with campaign and user data and validates existence of this url in project.
+        Checking if it was configured by the user.
+        """
+        if not self.drip_base.drip_model.campaign:
+            return None
+        email_token = EmailToken(self.user)
+        campaign_uidb64, uidb64, token = email_token.get_uidb64_token(self.drip_base.drip_model.campaign.pk)
+        url_args = {"campaign_uidb64": campaign_uidb64, "uidb64": uidb64, "token": token}
+        unsubscribe_link = validate_path_existence("unsubscribe_campaign", url_args)
+        return unsubscribe_link
+
+    def _get_unsubscribe_link(self) -> Optional[str]:
+        """
+        Generate url for Unsubcribe to app with user data and validates existence of this url in project.
+        Checking if it was configured by the user.
+        """
+        email_token = EmailToken(self.user)
+        uidb64, token = email_token.get_uidb64_token_user_only()
+        url_args = {"uidb64": uidb64, "token": token}
+        unsubscribe_link = validate_path_existence("unsubscribe_app", url_args)
+        return unsubscribe_link
+
 
 class DripBase(object):
     """
@@ -343,6 +397,14 @@ class DripBase(object):
             ).distinct()
         return self._queryset
 
+    def get_drip_unsubscribe_users_config(self) -> bool:
+        drip_unsubscribe_users_config = getattr(
+            settings,
+            "DRIP_UNSUBSCRIBE_USERS",
+            False,
+        )
+        return drip_unsubscribe_users_config
+
     def run(self) -> Optional[int]:
         """Get the queryset, prune sent people, and send it.
 
@@ -357,15 +419,56 @@ class DripBase(object):
 
         return count
 
+    def exclude_unsubcribed_users_drip(self) -> None:
+        """If DRIP_UNSUBSCRIBE_USERS is set to True, get a list of unsubscribed users ids to Drip model."""
+        unsubscribed_ids: Union[QuerySet, List] = []
+        unsubscribe_users = self.get_drip_unsubscribe_users_config()
+        if unsubscribe_users:
+            # Unsubscribed users on Drip
+            unsubscribed_ids = self.drip_model.unsubscribed_users.all().values_list("id", flat=True)
+        self._queryset = self.get_queryset().exclude(id__in=unsubscribed_ids)
+
+    def exclude_unsubcribed_users_campaign(self) -> None:
+        """If DRIP_UNSUBSCRIBE_USERS is set to True, get a list of unsubscribed users ids to Campaign model."""
+        unsubscribed_ids: Union[QuerySet, List] = []
+        unsubscribe_users = self.get_drip_unsubscribe_users_config()
+        campaign = self.drip_model.campaign
+        if unsubscribe_users and campaign:
+            # Unsubscribed users on Campaign
+            unsubscribed_ids = campaign.unsubscribed_users.all().values_list("id", flat=True)
+        self._queryset = self.get_queryset().exclude(id__in=unsubscribed_ids)
+
+    def exclude_unsubcribed_users_general(self) -> None:
+        """If DRIP_UNSUBSCRIBE_USERS is set to True, get a list of unsubscribed users ids to app (all emails)."""
+        unsubscribed_ids: Union[QuerySet, List] = []
+        unsubscribe_users = self.get_drip_unsubscribe_users_config()
+        if unsubscribe_users:
+            # Unsubscribed users on Drip
+            unsubscribed_ids = UserUnsubscribe.objects.all().values_list("user__id", flat=True)
+        self._queryset = self.get_queryset().exclude(id__in=unsubscribed_ids)
+
+    def exclude_sent_drips_users(self) -> None:
+        """If configured in can_resend_drip changes queryset excluding Users ids who have a SentDrip already."""
+        if not self.drip_model.can_resend_drip:
+            target_user_ids = self.get_queryset().values_list("id", flat=True)
+            exclude_user_ids = SentDrip.objects.filter(
+                date__lt=conditional_now(),
+                drip=self.drip_model,
+                user__id__in=target_user_ids,
+            ).values_list("user_id", flat=True)
+            self._queryset = self.get_queryset().exclude(id__in=exclude_user_ids)
+
     def prune(self) -> None:
-        """Do an exclude for all Users who have a SentDrip already."""
-        target_user_ids = self.get_queryset().values_list("id", flat=True)
-        exclude_user_ids = SentDrip.objects.filter(
-            date__lt=conditional_now(),
-            drip=self.drip_model,
-            user__id__in=target_user_ids,
-        ).values_list("user_id", flat=True)
-        self._queryset = self.get_queryset().exclude(id__in=exclude_user_ids)
+        """Do an exclude for all Users who have a SentDrip already and if configured the unsubscribed users."""
+        self.get_queryset()
+        # sent drips exclude
+        self.exclude_sent_drips_users()
+        # unsubscribed users exclude from Drip
+        self.exclude_unsubcribed_users_drip()
+        # unsubscribed users exclude from Campaign
+        self.exclude_unsubcribed_users_campaign()
+        # unsubscribed users exclude from all emails
+        self.exclude_unsubcribed_users_general()
 
     def get_count_from_queryset(self, message_class) -> int:
         """
