@@ -2,20 +2,22 @@ from apscheduler.schedulers.background import BackgroundScheduler
 from django.conf import settings
 from django.core.management import call_command
 
-DRIP_SCHEDULE_SETTINGS = getattr(settings, "DRIP_SCHEDULE_SETTINGS", {})
-
-
-DRIP_SCHEDULE = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE", False)
-DRIP_SCHEDULE_DAY_OF_WEEK = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE_DAY_OF_WEEK", 0)
-DRIP_SCHEDULE_HOUR = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE_HOUR", 0)
-DRIP_SCHEDULE_MINUTE = DRIP_SCHEDULE_SETTINGS.get("DRIP_SCHEDULE_MINUTE", 0)
+from drip.scheduler.constants import SCHEDULER_CRON, get_drip_scheduler_settings
 
 
 def cron_send_drips():
     def call_send_drips_command():
         call_command("send_drips")
 
-    if DRIP_SCHEDULE:
+    (
+        DRIP_SCHEDULE,
+        DRIP_SCHEDULE_DAY_OF_WEEK,
+        DRIP_SCHEDULE_HOUR,
+        DRIP_SCHEDULE_MINUTE,
+        SCHEDULER,
+    ) = get_drip_scheduler_settings()
+    CRON_ENABLED = SCHEDULER == SCHEDULER_CRON
+    if DRIP_SCHEDULE and CRON_ENABLED:
         cron_scheduler = BackgroundScheduler()
         cron_scheduler.add_job(
             call_send_drips_command,
@@ -26,3 +28,4 @@ def cron_send_drips():
             timezone=getattr(settings, "TIME_ZONE", "UTC"),
         )
         cron_scheduler.start()
+        return cron_scheduler
